#include <cmath>
#include <SDL/SDL.h>

#include "cMaze.h"
#include "cMazeField.h"
#include "constants.h"
#include "functions.h"
#include "globals.h"

int cMaze::dAddFront(int x, int y) {
    if (!(x >= maze->gSize() || y >= maze->gSize())) {
            std::vector<int> d;
            d.push_back(x);
            d.push_back(y);

            dFronts.push_back(d);
    }

    return 0;
}

int cMaze::pathfinding() {
    int iX = 0, iY = 0;
    // std::vector< std::vector <int> >::iterator i;
    int i;
    int tx, ty;
    bool stepped = false;
    std::vector<int> tVector;

    switch (pathState) {
        case PATH_FIND:
            while (!stepped) {
                i = 0;

                if (i == dFronts.size()) { // EMPTY
                    stepped = true;
                    pathState = PATH_BACK;
                } else {
                    iX = dFronts[i][0];
                    iY = dFronts[i][1];

                    if (iX == maze->gSize() - 1 && iY == maze->gSize() - 1) {
                        pathState = PATH_BACK; // FOUND
                        stepped = true;
                        dCurX = iX;
                        dCurY = iY;
                        maze->fillD();
                    } else { // EXPAND
                        if (iY > 0) { // TOP
                            if (maze->gWall(iX, iY, CELL_TOP) == C_NOWALL && !maze->isVisitedD(iX, iY - 1)) {
                                dAddFront(iX, iY - 1);
                                maze->visitD(iX, iY - 1);
                                maze->sMom(iX, iY - 1, iX, iY);
                                stepped = true;
                            }
                        }

                        if (iX < maze->gSize() - 1) { // RIGHT
                            if (maze->gWall(iX, iY, CELL_RIGHT) == C_NOWALL && !maze->isVisitedD(iX + 1, iY)) {
                                dAddFront(iX + 1, iY);
                                maze->visitD(iX + 1, iY);
                                maze->sMom(iX + 1, iY, iX, iY);
                                stepped = true;
                            }
                        }

                        if (iY < maze->gSize() - 1) { // BOTTOM
                            if (maze->gWall(iX, iY, CELL_BOTTOM) == C_NOWALL && !maze->isVisitedD(iX, iY + 1)) {
                                dAddFront(iX, iY + 1);
                                maze->visitD(iX, iY + 1);
                                maze->sMom(iX, iY + 1, iX, iY);
                                stepped = true;
                            }
                        }

                        if (iX > 0) { // LEFT
                            if (maze->gWall(iX, iY, CELL_LEFT) == C_NOWALL && !maze->isVisitedD(iX - 1, iY)) {
                                dAddFront(iX - 1, iY);
                                maze->visitD(iX - 1, iY);
                                maze->sMom(iX - 1, iY, iX, iY);
                                stepped = true;
                            }
                        }

                        dFronts.erase(dFronts.begin() + i);
                    }
                }
            }
            break;
        case PATH_BACK:
            if (dCurX == 0 && dCurY == 0) {
                tVector.push_back(dCurX);
                tVector.push_back(dCurY);
                dPath.push_back(tVector);

                maze->unvisitD(dCurX, dCurY);
                maze->invertD();
                mazeState = MAZE_DONE;
            } else {
                tVector.push_back(dCurX);
                tVector.push_back(dCurY);
                dPath.push_back(tVector);

                maze->unvisitD(dCurX, dCurY);
                tx = maze->gMomX(dCurX, dCurY);
                ty = maze->gMomY(dCurX, dCurY);
                dCurX = tx; dCurY = ty;
            }

            // mazeState == MAZE_DONE;
            break;
        case PATH_FINISH:

            break;
    }

    return 0;
}

int cMaze::render(bool dPath) {
    SDL_Surface* tSurf;
    float factor;

    tSurf = maze->render(dPath);

    switch (maze->gSize()) {
        case 5:
            factor = 10;
            break;
        case 25:
            factor = 2;
            break;
        case 50:
            applySurface(tSurf, screen, 0, 50);
            return 0;
            break;
    }

    enlarge(tSurf, surfEnlarged, factor); // <- HIER WAS IK MEE BEZIG

    applySurface(surfEnlarged, screen, 0, 50);

    return 0;
}

int cMaze::logic(int tempo, bool path) {
    int deltaT;

    /*
        switch (tempo) {
            case 1:
                deltaT = 500;
                break;
            case 2:
                deltaT = 100;
                break;
            case 3:
                deltaT = 1;
                break;
            default:
                deltaT = 100;
        */

    if (tempo > 1) {
            deltaT = pow(tempo - 9, 2) * (1000 / 49.0);
    }

    if (SDL_GetTicks() - lastFrame >= deltaT && tempo > 1) {
        switch (mazeState) {
            case MAZE_MGNR:
                this->generateStep();
                break;
            case MAZE_PATH:
                this->pathfinding();
                break;
            case MAZE_DONE:
                maze->fillD();
                maze->invertD();
                if (path) {
                    for (int i = 0; i < dPath.size(); i++) {
                        maze->visitD(dPath[i][0], dPath[i][1]);
                    }
                }
                break;
        }

        lastFrame = SDL_GetTicks();
    }

    return mazeState;
}
