#include <SDL/SDL_ttf.h>

#include "button.h"
#include "constants.h"
#include "functions.h"
#include "gamestates.h"
#include "globals.h"

int gsParams::logic() {
    if (bBack->logic(mX, mY, mouseState) == B_CLICK) {
        gameMan->setNextState(STATE_MENU);
    }

    algorithm->logic(mX, mY, mouseState);
    size->logic(mX, mY, mouseState);
    speed->logic(mX, mY, mouseState);

    if (bGenerate->logic(mX, mY, mouseState) == B_CLICK) {
        /*
        algorithmP = algorithm->getSelected();
        sizeP = size->getSelected();
        speedP = speed->getSelected(); // For checking if the checkboxes work
        */

        gameMan->params[PARAM_ALGO] = algorithm->getSelected(); // max 4
        gameMan->params[PARAM_SIZE] = size->getSelected(); // 3
        gameMan->params[PARAM_SPD] = speed->getSelected(); // 3


        gameMan->setNextState(STATE_MAZE);
    }

    return 0;
}

int gsParams::render() {
    // SDL_Surface* dummyS;

    SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, 255, 255, 255));

    bBack->render();
    bGenerate->render();

    algorithm->render();
    size->render();
    speed->render();

    // Text
    applySurface(instructions, screen, 262, 385);

    applySurface(header, screen, SCR_W - header->w - 15, 10);

    applySurface(algorithmHeader, screen, 420 - algorithmHeader->w, 90); // -30
    applySurface(algorithmMuur, screen, 420 - algorithmMuur->w, 150);
    applySurface(algorithmHunt, screen, 420 - algorithmHunt->w, 202);
    applySurface(algorithmBinair, screen, 420 - algorithmBinair->w, 250);
    applySurface(algorithmPrims, screen, 420 - algorithmPrims->w, 300);

    applySurface(sizeHeader, screen, 160 - sizeHeader->w, 90);
    applySurface(sizeSmall, screen, 160 - sizeSmall->w, 150);
    applySurface(sizeMiddle, screen, 160 - sizeMiddle->w, 201);
    applySurface(sizeBig, screen, 160 - sizeBig->w, 251);

    applySurface(speedHeader, screen, 160 - speedHeader->w, 295);
    applySurface(speedSlow, screen, 160 - speedSlow->w, 355);
    applySurface(speedMiddle, screen, 160 - speedMiddle->w, 405);
    applySurface(speedFast, screen, 160 - speedFast->w, 455);

    /*
    switch(gameMan->params[PARAM_ALGO]) {
        case 0:
            dummyS = zero;
            break;
        case 1:
            dummyS = one;
            break;
        case 2:
            dummyS = two;
            break;
        case 3:
            dummyS = three;
            break;
        case 4:
            dummyS = four;
            break;
    }

    applySurface(dummyS, screen, 0, 0);

    switch(gameMan->params[PARAM_SIZE]) {
        case 0:
            dummyS = zero;
            break;
        case 1:
            dummyS = one;
            break;
        case 2:
            dummyS = two;
            break;
        case 3:
            dummyS = three;
            break;
        case 4:
            dummyS = four;
            break;
    }

    applySurface(dummyS, screen, 50, 0);

    switch(gameMan->params[PARAM_SPD]) {
        case 0:
            dummyS = zero;
            break;
        case 1:
            dummyS = one;
            break;
        case 2:
            dummyS = two;
            break;
        case 3:
            dummyS = three;
            break;
        case 4:
            dummyS = four;
            break;
    }

    applySurface(dummyS, screen, 100, 0); // For checking if checkboxes work
    */

    SDL_Flip(screen);
    return 0;
}

gsParams::gsParams() {
    bBack = new cButton("Terug", 20, 500, false, fNormal, B_VISIBLE, B_STANDARD);
    bGenerate = new cButton("Genereer", 355, 500, false, fNormal, B_VISIBLE, B_STANDARD);

    algorithm = new cCheckboxes(440, 158, 50, false, 4, 20, 2);
    size = new cCheckboxes(180, 158, 50, false, 3, 20, 2);
    speed = new cCheckboxes(180, 362, 50, false, 3, 20, 2);

    algorithm->setSelected(3);
    size->setSelected(3);
    speed->setSelected(2);

    TTF_Font* fStd = fSmall;
    TTF_Font* fKopje = fMiniHeader;

    instructions = loadImage("Media/instructies.png");

    header = TTF_RenderText_Shaded(fHeader, "Genereer doolhof", cBlack, cWhite);

    algorithmHeader = TTF_RenderText_Shaded(fKopje, "Algoritme", cBlack, cWhite);
    algorithmMuur = TTF_RenderText_Shaded(fStd, "Muurgroei", cBlack, cWhite);
    algorithmHunt = TTF_RenderText_Shaded(fStd, "Hunt & Kill", cBlack, cWhite);
    algorithmBinair = TTF_RenderText_Shaded(fStd, "Binair", cBlack, cWhite);
    algorithmPrims = TTF_RenderText_Shaded(fStd, "Prim's", cBlack, cWhite);

    sizeHeader = TTF_RenderText_Shaded(fKopje, "Grootte", cBlack, cWhite);
    sizeSmall = TTF_RenderText_Shaded(fStd, "Klein", cBlack, cWhite);
    sizeMiddle = TTF_RenderText_Shaded(fStd, "Normaal", cBlack, cWhite);
    sizeBig = TTF_RenderText_Shaded(fStd, "Groot", cBlack, cWhite);

    speedHeader = TTF_RenderText_Shaded(fKopje, "Tempo", cBlack, cWhite);
    speedSlow = TTF_RenderText_Shaded(fStd, "Langzaam", cBlack, cWhite);
    speedMiddle = TTF_RenderText_Shaded(fStd, "Normaal", cBlack, cWhite);
    speedFast = TTF_RenderText_Shaded(fStd, "Snel", cBlack, cWhite);

    zero = TTF_RenderText_Shaded(fSmall, "0", cBlack, cWhite);
    one = TTF_RenderText_Shaded(fSmall, "1", cBlack, cWhite);
    two = TTF_RenderText_Shaded(fSmall, "2", cBlack, cWhite);
    three = TTF_RenderText_Shaded(fSmall, "3", cBlack, cWhite);
    four = TTF_RenderText_Shaded(fSmall, "4", cBlack, cWhite);

    algorithmP = 0;
    sizeP = 0;
    speedP = 0;
}

gsParams::~gsParams() {
    delete bBack;
    delete bGenerate;

    delete algorithm;
    delete size;
    delete speed;

    SDL_FreeSurface(instructions);

    SDL_FreeSurface(header);

    SDL_FreeSurface(algorithmHeader);
    SDL_FreeSurface(algorithmMuur);
    SDL_FreeSurface(algorithmHunt);
    SDL_FreeSurface(algorithmBinair);
    SDL_FreeSurface(algorithmPrims);

    SDL_FreeSurface(sizeHeader);
    SDL_FreeSurface(sizeSmall);
    SDL_FreeSurface(sizeMiddle);
    SDL_FreeSurface(sizeBig);

    SDL_FreeSurface(speedHeader);
    SDL_FreeSurface(speedSlow);
    SDL_FreeSurface(speedMiddle);
    SDL_FreeSurface(speedFast);

    SDL_FreeSurface(zero);
    SDL_FreeSurface(one);
    SDL_FreeSurface(two);
    SDL_FreeSurface(three);
    SDL_FreeSurface(four);
}
