#include <cmath>
#include <vector>
#include "mtwist.h"

#include "cMazeField.h"
#include "cMazeCell.h"
#include "constants.h"
#include "functions.h"
#include "globals.h"

int cMazeField::useRow(int rowN) {
    for (int x = 0; x < size; x++) {
        use(x, rowN);
    }

    return 0;
}

int cMazeField::rowHasUnvisited(int rowN) {
    for (int x = 0; x < size; x++) {
        if (isVisited(x, rowN) == false) {
            return x;
        }
    }

    return -1;
}

int cMazeField::connect(int x1, int y1, int x2, int y2, bool p) {
    /*if (pNCX > pFX) {
                    maze->sWall(pNCX, pNCY, CELL_LEFT, C_NOWALL);
                } else if (pNCX < pFX) {
                    maze->sWall(pNCX, pNCY, CELL_RIGHT, C_NOWALL);
                } else if (pNCY > pFY) {
                    maze->sWall(pNCX, pNCY, CELL_TOP, C_NOWALL);
                } else if (pNCY < pFY) {
                    maze->sWall(pNCX, pNCY, CELL_BOTTOM, C_NOWALL);
                }
            */
    if (!(inRange(x1, 0, size) && inRange(x2, 0, size) && inRange(y1, 0, size) && inRange(y2, 0, size))) {
        return 1;
    } else if (x1 == x2 && y1 == y2) {
        return 2;
    } else if (abs(x1 - x2) > 1 || abs(y1 - y2) > 1 || (abs(x1 - x2) >= 1 && abs(y1 - y2) >= 1 )) {
        return 3;
    }

    if (x1 < x2) {
        sWall(x1, y1, CELL_RIGHT, p);
    } else if (x1 > x2) {
        sWall(x1, y1, CELL_LEFT, p);
    } else if (y1 < y2) {
        sWall(x1, y1, CELL_BOTTOM, p);
    } else if (y1 > y2) {
        sWall(x1, y1, CELL_TOP, p);
    }

    return 0;
}

bool cMazeField::hasUnvisitedN(int x, int y) {
    int n1x = x - 1; // left neighbour
    int n1y = y;

    int n2x = x + 1; // right neighbour
    int n2y = y;

    int n3x = x; // upper neighbour
    int n3y = y-1;

    int n4x = x; // bottom neighbor
    int n4y = y + 1;

    if (inRange(n1x, 0, size) && inRange(n1y, 0, size)) { // check if inbound
        if (isVisited(n1x, n1y) == false) {
            return true;
        }
    }

    if (inRange(n2x, 0, size) && inRange(n2y, 0, size)) { // check if inbound
        if (isVisited(n2x, n2y) == false) {
            return true;
        }
    }

    if (inRange(n3x, 0, size) && inRange(n3y, 0, size)) { // check if inbound
        if (isVisited(n3x, n3y) == false) {
            return true;
        }
    }

    if (inRange(n4x, 0, size) && inRange(n4y, 0, size)) { // check if inbound
        if (isVisited(n4x, n4y) == false) {
            return true;
        }
    }

    return false;
}

int cMazeField::gSize() {
    return size;
}
// standaard instelling muren checken
bool cMazeField::gWall(int x, int y, int pos) {
    if (!(x >= size || y >= size)) {
        switch (pos) {
            case CELL_TOP:
                if (y == 0) {
                    return C_WALL;
                } else {
                    return maze[x][y].gTop(); // look at cell
                }

                break;
            case CELL_RIGHT:
                if (x == size - 1) {
                    return C_WALL;
                } else {
                    return maze[x+1][y].gLeft(); // look at right neighbour
                }
                break;
            case CELL_BOTTOM:
                if (y == size - 1) {
                    return C_WALL;
                } else {
                    return maze[x][y+1].gTop(); // look at below neighbour
                }
                break;
            case CELL_LEFT:
                if (x == 0) {
                    return C_WALL;
                } else {
                    return maze[x][y].gLeft(); // look at cell
                }

                break;
        }
    }

    return false;
}

bool cMazeField::isVisited(int x, int y) {
    if (!(x >= size || y >= size)) {
        return maze[x][y].gVisit();
    }

    return false;
}

int cMazeField::sWall(int x, int y, int pos, int p) {
    if (!(x >= size || y >= size)) {
        switch(pos) {
            case CELL_TOP:
                if (y != 0) {
                    maze[x][y].sTop(p);
                } else {
                    return 1;
                }
                break;
            case CELL_RIGHT:
                if (x != size - 1) {
                    maze[x+1][y].sLeft(p);
                } else {
                    return 1;
                }
                break;
            case CELL_BOTTOM:
                if (y != size - 1) {
                    maze[x][y+1].sTop(p);
                } else {
                    return 1;
                }
                break;
            case CELL_LEFT:
                if (x != 0) {
                    maze[x][y].sLeft(p);
                } else {
                    return 1;
                }
                break;
        }
    }

    return 0;
}

int cMazeField::use(int x, int y) {
    if (!(x >= size || y >= size)) {
        maze[x][y].use();
    }
}

int cMazeField::unuse() {
    for (int x = 0; x < size; x++) {
        for (int y = 0; y < size; y++) {
            maze[x][y].unuse();
        }
    }

    return 0;
}

int cMazeField::visitD(int x, int y) {
    if (!(x >= size || y >= size)) {
        maze[x][y].visitD();
    }
    return 0;
}

SDL_Surface* cMazeField::render() {
    SDL_FillRect(mazeSurface, NULL, SDL_MapRGB(mazeSurface->format, 255, 255, 255));

    for (int x = 0; x < size; x++) {
        for (int y = 0; y < size; y++) {
            maze[x][y].render(mazeSurface, this);
        }
    }

    return mazeSurface;
}

int cMazeField::visit(int x, int y) {
    if (!(x >= size || y >= size)) {
        maze[x][y].visit();
    }
    return 0;
}

cMazeField::cMazeField(int fSize) {
    size = fSize;
    maze.resize(fSize);

    for (int x = 0; x < maze.size(); x++) {
        for (int y = 0; y < fSize; y++) {
            maze[x].push_back(cMazeCell(x, y));
        }
    }

    /* BINARY MAZE
    for (int x = 0; x < fSize; x++) {
        for (int y = 0; y < fSize; y++) {
            if (mt_lrand() % 2) {
                if (x == this->gSize() - 1) {
                    this->sWall(x, y, CELL_BOTTOM, C_NOWALL);
                } else {
                    this->sWall(x, y, CELL_RIGHT, C_NOWALL);
                }
            } else {
                if (y == this->gSize() - 1) {
                    this->sWall(x, y, CELL_RIGHT, C_NOWALL);
                } else {
                    this->sWall(x, y, CELL_BOTTOM, C_NOWALL);
                }
            }
        }
    }
    */

    mazeSurface = SDL_CreateRGBSurface(SDL_SWSURFACE, fSize * 10, fSize * 10, screen->format->BitsPerPixel, screen->format->Rmask, screen->format->Gmask, screen->format->Bmask, screen->format->Amask);
}

cMazeField::~cMazeField() {
    for (int x = 0; x < size; x++) {
        for (int y = 0; y < size; y++) {

        }
        maze[x].clear();
    }
    maze.clear();

    SDL_FreeSurface(mazeSurface);
}
