#include <SDL/SDL.h>

#include "checkboxes.h"
#include "functions.h"
#include "globals.h"

int cCheckbox::logic(int mX, int mY, Uint8 mouseState) {
    bool prevState = state;
    int retVal = 0;

    if (pointInRect(mX, mY, outer)) {
        if (prevMouseState == 0) {
            if (SDL_BUTTON(SDL_BUTTON_LEFT)&mouseState) {
                state = !state;
            }
        } else {
            if (SDL_BUTTON(SDL_BUTTON_LEFT)&prevMouseState == false && SDL_BUTTON(SDL_BUTTON_LEFT)&mouseState == true) {
                state = !state;
            }
        }
    }

    if (prevState == CHECK_ON) { // Was initially on
        if (state == CHECK_ON) {
            // Nothing happened
            retVal = CHECK_NOCHANGE;
        } else {
            // Turned off
            retVal = CHECK_TURNOFF;
        }
    } else { // Was initially off
        if (state == CHECK_ON) {
            // Turned on
            retVal = CHECK_TURNON;
        } else {
            // Nothing happened
            retVal = CHECK_NOCHANGE;
        }
    }

    prevMouseState = mouseState;

    return retVal;
}

int cCheckbox::render() {
    SDL_FillRect(screen, &outer, 0);
    SDL_FillRect(screen, &middle, SDL_MapRGB(screen->format, 255, 255, 255));

    if (state == CHECK_ON) {
        SDL_FillRect(screen, &inner, 0);
    }

    return 0;
}

int cCheckbox::setState(bool newState) {
    state = newState;

    return 0;
}

bool cCheckbox::getState() {
    return state;
}

cCheckbox::cCheckbox(int x, int y, int width, int border, bool startState) {
    outer.x = x;
    outer.y = y;
    outer.w = width;
    outer.h = outer.w;

    middle.x = x + border;
    middle.y = y + border;
    middle.w = width - 2 * border;
    middle.h = middle.w;

    inner.x = x + 2 * border;
    inner.y = y + 2 * border;
    inner.w = width - 4 * border;
    inner.h = inner.w;

    state = startState;
    prevMouseState = 0;
}

cCheckbox::~cCheckbox() {

}
