
#include "cMazeCell.h"
#include "cMazeField.h"
#include "constants.h"
#include "functions.h"
#include "gamestates.h"
#include "globals.h"

int gsMaze::logic() {
    if (bBack->logic(mX, mY, mouseState) == B_CLICK) {
        gameMan->setNextState(STATE_MENU);
    }

    speed->logic(mX, mY, mouseState);

    maze->logic(speed->getSelected());

    return 0;
}

int gsMaze::render() {
    SDL_FillRect(screen, NULL, SDL_MapRGB(screen->format, 255, 255, 255));

    applySurface(tagSpeed, screen, 260, 0);
    applySurface(tagPause, screen, 120, 22);
    applySurface(tagFast, screen, 450, 22);

    bBack->render();

    speed->render();

    maze->render();

    SDL_Flip(screen);

    return 0;
}

gsMaze::gsMaze() {
    bBack = new cButton("Terug", 10, 5, false, fNormal, B_VISIBLE, B_STANDARD);

    tagPause = TTF_RenderText_Shaded(fTag, "Pause", cBlack, cWhite);
    tagFast = TTF_RenderText_Shaded(fTag, "Snel", cBlack, cWhite);
    tagSpeed = TTF_RenderText_Shaded(fTag, "Snelheid", cBlack, cWhite);

    speed = new cCheckboxes( 183, 25, 30, true, MAXSPEED, 20, 2);

    int mazeSize;
    switch (gameMan->params[PARAM_SIZE]) {
        case 1:
            // maze = new cBinary(5);
            mazeSize = 5;
            break;
        case 2:
            // maze = new cBinary(20);
            mazeSize = 25;
            break;
        case 3:
            // maze = new cBinary(50);
            mazeSize = 50;
            break;
        default:
            // maze = new cBinary(50);
            mazeSize = 50;
            break;
    }

    switch (gameMan->params[PARAM_ALGO]) {
        case 1:
            maze = new cBinary(mazeSize);
            break;
        case 2:
            maze = new cHuntKill(mazeSize);
            break;
        case 3:
            maze = new cBinary(mazeSize);
            break;
        case 4:
            maze = new cPrims(mazeSize);
            break;
    }

    switch (gameMan->params[PARAM_SPD]) {
        case 1:
            speed->setSelected(2);
            break;
        case 2:
            speed->setSelected(8);
            break;
        case 3:
            speed->setSelected(9);
            break;
    }
}

gsMaze::~gsMaze() {
    delete bBack;
    delete maze;
    delete speed;

    SDL_FreeSurface(tagPause);
    SDL_FreeSurface(tagFast);
    SDL_FreeSurface(tagSpeed);
}
